#!/usr/bin/sed -nrsf

:_Problem
	# sokoban derivative (only the physics system)
	# https://cg.esolangs.gay/35/
:_Restrictions
	# Tile values are only in [0-9] range and board size is fixed at 6x6.


:main
	#test if GNU extensions are supported
	1v

	#store input data, then parse the moves (first line) and the board (remaining 6 lines)
	H
	1{
		/[WASD]/!{
			s/.*/Error: no moves, [WASD]+, found on line 1!/
			p;q
		}
	}
	2,${
		/^(-?[0-9][ \t]*){6}$/!{
			s/.*/Error: board size must be 6x6, with each tile having a value in [0-9] range!/
			p;q
		}
	}
	$!d
	g
	s:^\n[^\n]+::
	/(\n[^\n]+){6}/!{
		s/.*/Error: board size must be 6x6!/
		p;q
	}
	s:[0-9 \t\n]::g
	/^-$/!{
		s/.*/Error: exactly one tile, the player, must have its value prepended with '-' (negative value)!/
		p;q
	}
	x
	s: ::g

	#print input state
	s/^\n/\nInput state:\n()/p
	s:^\n[^\n]+\n\(\)::
	x

	s/.*/Intermediary states:/p
	:loop
		g
		#call one_move, save resulting state, then discard the move
		b one_move
		:return
			h;x
			s:^[WASD]::
			x
		#print intermediary state (skip if output state reached) and loop again if needed
		s:^[WASD]:(&):
		/^\([WASD]\)\n/!p
	/^\([WASD]\)\n/!b loop

	#print output state
	s/^/Output state:\n/p
b EOS


:one_move
	#index the board
	s:\n(-?[0-9])(-?[0-9])(-?[0-9])(-?[0-9])(-?[0-9])(-?[0-9])\n:\n,0=\1,1=\2,2=\3,3=\4,4=\5,5=\6\n@:
	s:@(-?[0-9])(-?[0-9])(-?[0-9])(-?[0-9])(-?[0-9])(-?[0-9])\n:,6=\1,7=\2,8=\3,9=\4,10=\5,11=\6\n@:
	s:@(-?[0-9])(-?[0-9])(-?[0-9])(-?[0-9])(-?[0-9])(-?[0-9])\n:,12=\1,13=\2,14=\3,15=\4,16=\5,17=\6\n@:
	s:@(-?[0-9])(-?[0-9])(-?[0-9])(-?[0-9])(-?[0-9])(-?[0-9])\n:,18=\1,19=\2,20=\3,21=\4,22=\5,23=\6\n@:
	s:@(-?[0-9])(-?[0-9])(-?[0-9])(-?[0-9])(-?[0-9])(-?[0-9])\n:,24=\1,25=\2,26=\3,27=\4,28=\5,29=\6\n@:
	s:@(-?[0-9])(-?[0-9])(-?[0-9])(-?[0-9])(-?[0-9])(-?[0-9]):,30=\1,31=\2,32=\3,33=\4,34=\5,35=\6:

	#setup number line and the tile adjacency table (lazy to calculate it)
        s:$:\nL0112233445566778899L:
	s:-([0-9])(.*L[0-9]+)\1\1:-\1\2\1-\1:
	s:$:<ADJ>,0-S6D1,1-S7A0D2,2-S8A1D3,3-S9A2D4,4-S10A3D5,5-S11A4:
	s:$:,6-W0S12D7,7-W1S13A6D8,8-W2S14A7D9,9-W3S15A8D10,10-W4S16A9D11,11-W5S17A10:
	s:$:,12-W6S18D13,13-W7S19A12D14,14-W8S20A13D15,15-W9S21A14D16,16-W10S22A15D17,17-W11S23A16:
	s:$:,18-W12S24D19,19-W13S25A18D20,20-W14S26A19D21,21-W15S27A20D22,22-W16S28A21D23,23-W17S29A22:
	s:$:,24-W18S30D25,25-W19S31A24D26,26-W20S32A25D27,27-W21S33A26D28,28-W22S34A27D29,29-W23S35A28:
	s:$:,30-W24D31,31-W25A30D32,32-W26A31D33,33-W27A32D34,34-W28A33D35,35-W29A34,<JDA>:

	#skip current move since player is already at the edge of the board
	/^([WASD]).*,([0-9][0-9]?)=-.*<ADJ>.*,\2-[^,]*\1/!{
		s/^[WASD]/Warning: current move & is skipped since the player is already at the edge of the board!\n&/
		P
		b skip
	}

	s:=-:=+-:
	:construct_path
		#from current player/box get the next tile in direction of current move
		s:^([WASD])(.*,)([0-9][0-9]?)(=\+.*<ADJ>.*,)\3(-[^,]*)\1([0-9][0-9]?):\6\n&:
		s:=\+:=@:
		#if next tile is an empty space, the path of tiles to move was finished, call apply_move
		/^([0-9][0-9]?)\n.*,\1=0/b apply_move
		#if next tile is a wall, skip current move
		/^([0-9][0-9]?)\n.*,\1=([0-9]).*L[0-9]+\2[0-9]*-/!{
			s/^[^\n]+\n([WASD])/Warning: current move \1 is skipped because of a blocking wall!\n\1/
			P
			b skip
		}
		#next tile is a box, mark it with + to become new current box from now on
		s:^([0-9][0-9]?)(\n.*,)\1=([0-9])(.*L[0-9]+)\3([0-9]*-):\1\2\1=+\3\4\3\5:
		s:^[^\n]+\n::
		#skip current move since current box is already at the edge of the board
		/^([WASD]).*,([0-9][0-9]?)=\+.*<ADJ>.*,\2-[^,]*\1/!{
			s/^[WASD]/Warning: current move & is skipped since last moveable box is already at the edge of the board!\n&/
			P
			b skip
		}
	b construct_path

	:apply_move
		#transform current move into opposite move, to go from empty space back to player
		s:\n[WASD]:& ,W->S,S->W,A->D,D->A :
		s:\n([WASD]) [^\n]*,\1->([WASD])[^\n]* :\n\2\n\1:

		:shift
			#call skip to finish, if current tile is already at the edge of the board
			/^([0-9][0-9]?)\n([WASD]).*<ADJ>.*,\1-[^,]*\2/!{
				s:^[^\n]+\n::
				b skip
			}
			#from current tile get the next tile in opposite direction of current move
			s:^([0-9][0-9]?)\n([WASD])(.*<ADJ>.*,)\1(-[^,]*)\2([0-9][0-9]?):\1 \5\n\2\3\1\4\2\5:
			#if next tile isn't marked with @, path was already shifted, call skip to finish
			/^[0-9][0-9]? ([0-9][0-9]?)\n.*,\1=@/!{
				s:^[^\n]+\n::
				b skip
			}
			#swap values of current tile and next tile, with next tile to become new current tile
			s:^([0-9][0-9]?) ([0-9][0-9]?)(\n.*,)\1=0(.*,)\2=(@-?[0-9]):\2\3\1=\5\4\2=0:
			s:^([0-9][0-9]?) ([0-9][0-9]?)(\n.*,)\2=(@-?[0-9])(.*,)\1=0:\2\3\2=0\5\1=\4:
		b shift

	:skip
		s:^[^\n]+\n::
		#strip tile markings
		s:=@:=:g
		s:=\+:=:g
		#remove the number line and the tile adjacency table, then strip the board index (for pretty printing back in main)
		s:\nL.*$::
		s:,[0-9][0-9]?=::g
b return


:EOS
	#End Of Script (mainly used to skip over remaining code, when needed)
